// ================================================
// Buttons
// ================================================
function Button(element, imageDefault, imageClicked, imageAlt) {
	LOG__("Object: Button");
	this.element = element;
	this.imageDefault = imageDefault;
	this.imageClicked = imageClicked;
	this.imageAlt     = imageAlt;

	// Press the button
	this.pressed = false;
	this.pressTime;
	this.pressTimerID;
	this.pressTimeLimit = 1 * 1000;
	this.pressRepeatInterval = 0;
	this.downWithShift = false;

    // Set Images
	this.setImageDefault(imageDefault);
	this.setImageClicked(imageClicked);
	this.setImageAlt(imageAlt);
	this.element.style.appleDashboardRegion = "dashboard-region(control rectangle 0 0 0 0)";
	
	// Events
	var localThis = this;
	this.mouseOverHandler  = function(theEvent) { localThis.buttonOver(theEvent); };
	this.mouseMoveHandler  = function(theEvent) { localThis.buttonMove(theEvent); };
	this.mouseDownHandler  = function(theEvent) { localThis.buttonDown(theEvent); };
	this.mouseUpHandler    = function(theEvent) { localThis.buttonUp(theEvent);   };
	this.mouseOutHandler   = function(theEvent) { localThis.buttonOut(theEvent);  };
	this.pressTimer        = function() { localThis.pressActionCall(); };
	this.element.addEventListener("mouseover", this.mouseOverHandler, false);
	this.element.addEventListener("mousedown", this.mouseDownHandler, false);
}

// Set Image
Button.prototype.setImageDefault = function(imageDefault) {
    this.imageDefault = imageDefault;
	this.element.setAttribute("src", this.imageDefault);    
}

Button.prototype.setImageClicked = function(imageClicked) {
    this.imageClicked = imageClicked;
}

Button.prototype.setImageAlt = function(imageAlt) {
    this.imageAlt = imageAlt;
}

// Mouse over
Button.prototype.buttonOver = function(theEvent) {
	LOG2__("Button.buttonOver");
	if (this.checkPositionCall(theEvent)) {
        this.overAction();
        this.element.addEventListener("mousemove", this.mouseOverHandler, false);
		this.element.addEventListener("mouseout",  this.mouseOutHandler,  false);
	}
	theEvent.stopPropagation();
	theEvent.preventDefault();
}


// Mouse move
Button.prototype.buttonMove = function(theEvent) {
	LOG2__("Button.buttonMove");
	if (this.checkPositionCall(theEvent)) {
        this.overAction();
	}
	theEvent.stopPropagation();
	theEvent.preventDefault();
}


// Mouse down
Button.prototype.buttonDown = function(theEvent) {
	LOG2__("Button.buttonDown");
	if (this.checkPositionCall(theEvent)) {
		this.downWithShift = theEvent.shiftKey;
		this.setPressed(true);
		this.element.addEventListener("mouseup",   this.mouseUpHandler,   false);
		this.element.addEventListener("mouseout",  this.mouseOutHandler,  false);
	}
	theEvent.stopPropagation();
	theEvent.preventDefault();
}

// Mouse up, Action
Button.prototype.buttonUp = function(theEvent) {
	LOG2__("Button.buttonUp");
	if (this.setPressed) {
		if (!theEvent.shiftKey)
			this.action();
		else
			this.shiftAction();
	}
	this.setPressed(false);
	this.element.removeEventListener("mouseup",   this.mouseUpHandler,   false);
	this.element.removeEventListener("mouseout",  this.mouseOutHandler,  false);

	theEvent.stopPropagation();
	theEvent.preventDefault();
}

// Mouse out
Button.prototype.buttonOut = function(theEvent) {
	LOG2__("Button.buttonOut");
    this.setPressed(false);
    this.outAction();
    this.element.removeEventListener("mouseup",   this.mouseUpHandler,   false);
    this.element.removeEventListener("mousemove", this.mouseUpHandler,   false);
    this.element.removeEventListener("mouseout",  this.mouseOutHandler,  false);

	theEvent.stopPropagation();
	theEvent.preventDefault();
}

// Set state of the button
Button.prototype.setPressed = function(flag) {
	LOG2__("Button.setPressed");
	if (flag == this.pressed)
		return;

	if (flag) {
		if (this.imageClicked)
			this.element.setAttribute("src", this.imageClicked);
	} else {
		if (this.imageDefault)
			this.element.setAttribute("src", this.imageDefault);
	}

	// Press timer
	if (this.pressTimerID) {
		clearTimeout(this.pressTimerID);
		this.pressTimerID = null;
	}
	if (flag)
		this.pressTimerID = setTimeout(this.pressTimer, this.pressTimeLimit);

	this.pressed = flag;
}

Button.prototype.checkPositionCall = function(theEvent) {
	var x = theEvent.layerX;
	var y = theEvent.layerY;
	LOG2__("Button.checkPositionCall [" + x + ", " + y + "]");

	return(this.checkPosition(x, y));
}

Button.prototype.checkPosition = function(x, y) {
	return true;
}

Button.prototype.action = function() {
	LOG2__("Button.action");
}

Button.prototype.shiftAction = function() {
	LOG2__("Button.shiftAction");
	this.action();
}

Button.prototype.pressActionCall = function() {
	LOG2__("Button.pressActionCall");
	if (this.pressTimerID) {
		clearTimeout(this.pressTimerID);
		this.pressTimerID = null;
		
		if (0 < this.pressRepeatInterval)
			this.pressTimerID = setTimeout(this.pressTimer, this.pressRepeatInterval);
	}

	if (!(this.downWithShift))
		this.pressAction();
	else
		this.pressShiftAction();
}

Button.prototype.pressAction = function() {
	return null;
}

Button.prototype.pressShiftAction = function() {
	this.pressAction();
}

Button.prototype.overAction = function() {
	LOG2__("Button.overAction");
}

Button.prototype.outAction = function() {
	LOG2__("Button.outAction");
}

// Called from key event handler
Button.prototype.keyDown = function() {
	LOG2__("Button.keyDown");

	this.downWithShift = false;
	this.setPressed(true);
}

Button.prototype.keyUp = function() {
	LOG2__("Button.keyUp");

	if (this.setPressed)
		this.action();
	this.setPressed(false);
}

Button.prototype.keyDownShift = function() {
	LOG2__("Button.keyDownShift");

	this.downWithShift = true;
	this.setPressed(true);
}

Button.prototype.keyUpShift = function() {
	LOG2__("Button.keyUpShift");

	if (this.setPressed)
		this.shiftAction();
	this.setPressed(false);
}

// Declaring Control Regions
Button.prototype.setRegion = function(str) {
	this.element.style.appleDashboardRegion = "dashboard-region(" + str + ")";
}


