var timerID;
var zeroHour, alarmTime;
var keyBuffer;
var rewritingMode;
var alarmMode;
var display = "front";
var sound, keyEvents, timerGraph, notification;
var startButton, setButtonA, setButtonB;
var inc43Button, inc21Button, dec43Button, dec21Button;

// Alarm sound file and repeat times (0 to 65535)
var soundFile = "/System/Library/Sounds/Submarine.aiff";
var soundRepeat = 10;
var soundRepeatShort = 3;

// Skin Files
var jacketFiles = [ "Rd", "Yl", "Bl", "Or", "Wt", "Gr", "Bk", "Pk", "Br", "Gl" ];
var jacketSampleContext;
var widgetPref;
var prefPane;
var websiteURI = "http://www.cyanworks.net/dboardTClock.html"

// Widget Initialize
function init() {
	timerID = null;
	zeroHour = null;
	alarmTime = 0;
	rewritingMode = true;
	keyBuffer = null;
	alarmMode = false;

	widgetPref = new WidgetPreference();
	widgetPref.jacketsNum = jacketFiles.length;
	if (window.widget) {
		widget.onshow = onShow;
		widget.onhide = onHide;
		widget.onremove = onRemove;
	}

	// Sound
	sound = new Sound();
	sound.load(soundFile);

	initControlButtons();

	timerGraph = new TimerGraph();
	notification = new Notification();

	// Load preferences
	widgetPref.load();
	setAlarmMode(widgetPref.value("alarmMode"));
	setTimer(widgetPref.value("clock"));
	setJacket(widgetPref.value("jacket"));
    
    // Balloon
    rewritePresetBalloon("clockA", widgetPref.value("clockA"));
    rewritePresetBalloon("clockB", widgetPref.value("clockB"));

	// Prepare the preference section
	prefPane = new PreferencePane(widgetPref);

	// EventHandlers
	keyEvents = new KeyEvents();

	// Start the clock
	if (alarmMode)
		alarmStart();
	runInterval();
}

// Create buttons
function initControlButtons() {
	startButton = new Button(document.getElementById("startbutton"),
	  "Images/startButton0.png", "Images/startButton1.png", null);
	setButtonA  = new Button(document.getElementById("setbuttonA"),
	  "Images/setButton0.png",  "Images/setButton1.png",  null);
	setButtonB  = new Button(document.getElementById("setbuttonB"),
	  "Images/setButton0.png",  "Images/setButton1.png",  null);
	inc43Button = new Button(document.getElementById("inc43button"),
	  "Images/inc43Button0.png", "Images/inc43Button1.png", null);
	inc21Button = new Button(document.getElementById("inc21button"),
	  "Images/inc21Button0.png", "Images/inc21Button1.png", null);
	dec43Button = new Button(document.getElementById("dec43button"),
	  "Images/dec43Button0.png", "Images/dec43Button1.png", null);
	dec21Button = new Button(document.getElementById("dec21button"),
	  "Images/dec21Button0.png", "Images/dec21Button1.png", null);

	// Properties of Start/Stop button
	startButton.action      = function() { alarmSwitch(); };
	startButton.setRegion("control circle 0 0 0 0");
	startButton.checkPosition = function(x, y) {
		return(Math.pow((x - 21) * 14, 2) + Math.pow((14 - y) * 21, 2) <= Math.pow(14 * 21, 2));
	};

	// Properties of Preset button A
	setButtonA.action       = function() { loadPreset("clockA"); };
	setButtonA.pressAction  = function() { savePreset("clockA"); };
	setButtonA.overAction   = function() {
        var balloon = document.getElementById("balloonpreseta");
        balloon.style.visibility = "visible";
        balloon.style.zIndex = 5;
    };
	setButtonA.outAction   = function() {
        var balloon = document.getElementById("balloonpreseta");
        balloon.style.visibility = "hidden";
        balloon.style.zIndex = 0;
    };
	setButtonA.setRegion("control circle 0 0 0 0");
	setButtonA.checkPosition = function(x, y) {
		return(Math.pow(x - 10, 2) + Math.pow(10 - y, 2) <= Math.pow(10, 2));
	};

	// Properties of Preset button B
	setButtonB.action       = function() { loadPreset("clockB"); };
	setButtonB.pressAction  = function() { savePreset("clockB"); };
	setButtonB.overAction   = function() {
        var balloon = document.getElementById("balloonpresetb");
        balloon.style.visibility = "visible";
        balloon.style.zIndex = 5;
    };
	setButtonB.outAction   = function() {
        var balloon = document.getElementById("balloonpresetb");
        balloon.style.visibility = "hidden";
        balloon.style.zIndex = 0;
    };
	setButtonB.setRegion("control circle 0 0 0 0");
	setButtonB.checkPosition = function(x, y) {
		return(Math.pow(x - 10, 2) + Math.pow(10 - y, 2) <= Math.pow(10, 2));
	};

	// Properties of Increment button 43
	inc43Button.action      = function() { setTimer(alarmTime +     60 * 60 * 1000); };
	inc43Button.shiftAction = function() { setTimer(alarmTime + 6 * 60 * 60 * 1000); };
	inc43Button.pressAction      = inc43Button.action;
	inc43Button.pressShiftAction = inc43Button.shiftAction;
	inc43Button.pressRepeatInterval = 0.05 * 1000;
	inc43Button.setRegion("control circle 0 -43 -43 0");
	inc43Button.checkPosition = function(x, y) {
		return(Math.pow(x - 43, 2) + Math.pow(43 - y, 2) <= Math.pow(42, 2));
	};

	// Properties of Increment button 21
	inc21Button.action      = function() { setTimer(alarmTime +  1 * 60 *1000); };
	inc21Button.shiftAction = function() { setTimer(alarmTime + 10 * 60 *1000); };
	inc21Button.pressAction      = inc21Button.action;
	inc21Button.pressShiftAction = inc21Button.shiftAction;
	inc21Button.pressRepeatInterval = 0.05 * 1000;
	inc21Button.setRegion("control circle 0 0 -43 -43");
	inc21Button.checkPosition = function(x, y) {
		return(Math.pow(x, 2) + Math.pow(43 - y, 2) <= Math.pow(42, 2));
	};

	// Properties of Decrement button 43
	dec43Button.action      = function() { setTimer(alarmTime -     60 * 60 * 1000); };
	dec43Button.shiftAction = function() { setTimer(alarmTime - 6 * 60 * 60 * 1000); };
	dec43Button.pressAction      = dec43Button.action;
	dec43Button.pressShiftAction = dec43Button.shiftAction;
	dec43Button.pressRepeatInterval = 0.05 * 1000;
	dec43Button.setRegion("control circle -43 -43 0 0");
	dec43Button.checkPosition = function(x, y) {
		return(Math.pow(x - 43, 2) + Math.pow(0 - y, 2) <= Math.pow(42, 2));
	};

	// Properties of Decrement button 21
	dec21Button.action      = function() { setTimer(alarmTime -  1 * 60 *1000); };
	dec21Button.shiftAction = function() { setTimer(alarmTime - 10 * 60 *1000); };
	dec21Button.pressAction      = dec21Button.action;
	dec21Button.pressShiftAction = dec21Button.shiftAction;
	dec21Button.pressRepeatInterval = 0.05 * 1000;
	dec21Button.setRegion("control circle -43 0 0 -43");
	dec21Button.checkPosition = function(x, y) {
		return(Math.pow(x, 2) + Math.pow(0 - y, 2) <= Math.pow(42, 2));
	};
}

function onShow() {
	rewritingMode = true;
	runInterval();
}

function onHide() {
	rewritingMode = false;

	var saveWait = (1 * 1000 + Math.round(Math.random() * 1 * 1000)); // Wait 1 - 2 sec
	setTimeout(function(){ widgetPref.setValue(alarmTime, "clock"); }, saveWait);
}

function onRemove() {
	widgetPref.removeMyPref();
}

// Waked up from the sleep mode
function systemDidWake() {
	LOG__("systemDidWake");
	runInterval();
}

// Work Alarm
function alarmSwitch() {
	if (notification.flag) {
		notification.end();
		return;
	}
	keyBuffer = null;

	if (alarmMode)
		alarmStop();   // if Alarm working, turn off.
	else
		alarmStart();  // if Alarm sleeping, turn on.
}

function alarmStart() {
	notification.end();
	setAlarmMode(true);
}

function alarmStop() {
	notification.end();
	setAlarmMode(false);
}

function runInterval() {
	var interval;
	var now = (new Date).getTime();
	LOG2__("runInterval()");

	if (alarmMode && (zeroHour <= now)) {
		// It's Time!
		while(zeroHour <= now)
			zeroHour += 24*60*60*1000;
		notification.start();
	}

	if (rewritingMode)
		rewrite();

	// Pace maker
	clearTimeout(timerID);
	interval = 60*1000 - (new Date).getTime() % (60*1000);

	timerID = setTimeout("runInterval()", interval);
}

// ================================================
// Notification
// ================================================
function Notification() {
	LOG__("Object: Notification");

	this.flag = false;
	this.badge = document.getElementById("notification");
}

Notification.prototype.start = function() {
	LOG2__("Notification.start()");

	this.flag = true;
	this.badge.style.display = "block";
	sound.play();
}

Notification.prototype.end = function() {
	LOG2__("Notification.end()");

	this.flag = false;
	this.badge.style.display = "none";
	sound.stop();
}

// ================================================
// TimerGraph
// ================================================
function TimerGraph() {
	LOG__("Object: TimerGraph");

	this.clockHandCanvas = document.getElementById("clockHand");
	this.alarmHandCanvas = document.getElementById("alarmHand");
	this.time = 0;
	this.alarmTime = 0;
	this.alarmMode = false;

	this.hourHandImg = new Image();
	this.hourHandImg.src = "Images/hourHand.png";
	this.hourHandMaskImg = new Image();
	this.hourHandMaskImg.src = "Images/hourHandMask.png";

	// Events
	var localThis = this;
	this.drawClockImgLater = function() { localThis.drawClockImg(); };
}

TimerGraph.prototype.setTime = function(theTime) {
	this.time = theTime;
	this.drawClockImg();
}

TimerGraph.prototype.setAlarmTime = function(theTime) {
	this.alarmTime = theTime;
	this.drawAlarmImg();
}

TimerGraph.prototype.setAlarmMode = function(flag) {
	this.alarmMode = flag;
	this.alarmHandCanvas.style.opacity = (this.alarmMode)? 1.0 : 0;
}

TimerGraph.prototype.drawClockImg = function() {
	var ctxt = this.clockHandCanvas.getContext("2d");
	var angle = ((this.time % (24*60*60*1000)) / (24*60*60*1000))*2*Math.PI;

	// If images are not loaded
	if (!((this.hourHandImg.complete) && (this.hourHandMaskImg.complete)))
		setTimeout(this.drawClockImgLater, 100);

	ctxt.clearRect(0, 0, 84, 84);

	// Move to center
	ctxt.save();
	ctxt.translate(84/2, 84/2)

	// Draw a mask
	ctxt.save();
	ctxt.rotate(angle);
	ctxt.drawImage(this.hourHandMaskImg, -84/2, -84/2);
	ctxt.restore();

	// Draw the hours hand in a mask
	ctxt.globalCompositeOperation = "source-in";
	ctxt.drawImage(this.hourHandImg, -84/2, -84/2);

	ctxt.restore();
}

TimerGraph.prototype.drawAlarmImg = function() {
	var ctxt = this.alarmHandCanvas.getContext("2d");
	var angle = ((this.alarmTime % (24*60*60*1000)) / (24*60*60*1000))*2*Math.PI;

	ctxt.clearRect(0, 0, 84, 84);

	// Move to center
	ctxt.save();
	ctxt.translate(84/2, 84/2)

	// Draw the alarm hand
	ctxt.save();
	ctxt.rotate(-90/180*Math.PI);
	ctxt.lineWidth = 6;
	ctxt.strokeStyle = "rgba(255, 153, 0, 1.0)";
	ctxt.arc(0, 0, 38, angle - 0.08, angle - 0.015, 0);
	ctxt.stroke();
	ctxt.arc(0, 0, 38, angle + 0.015, angle + 0.08, 0);
	ctxt.stroke();
	ctxt.restore();

	ctxt.restore();
}

// Rewrite Clock
function rewrite() {
	var alarm = document.getElementById("alarm");
	var alarmHour = document.getElementById("alarmhour");
	var alarmMin = document.getElementById("alarmmin");
	var graph = document.getElementById("graph");
	var hour, min;
	var digit43, digit21;

	var dd = new Date();
	var nowHour = dd.getHours();
	var nowMin = dd.getMinutes();
	var clock = document.getElementById("clock");
	var clockHour = document.getElementById("clockhour");
	var clockMin = document.getElementById("clockmin");

	// Rewrite the clock
	clockHour.innerHTML = nowHour;
	clockMin.innerHTML  = (nowMin < 10)?  ("0" + nowMin)  : nowMin;
	timerGraph.setTime(nowHour*60*60*1000 + nowMin*60*1000);

	if (keyBuffer == null) {
		// Display alarm time
		hour = Math.floor(alarmTime/1000/60/60);
		min = Math.floor((alarmTime - hour*60*60*1000)/1000/60);
		alarmHour.innerHTML = hour;
		alarmMin.innerHTML  = (min < 10)?  ("0" + min) : min;
		timerGraph.setAlarmTime(hour*60*60*1000 + min*60*1000);
	} else {
		// Display numbers from keyboard
		digit43 = Math.floor((keyBuffer - Math.floor(keyBuffer/10000)*10000)/100);
		digit21 = keyBuffer - Math.floor(keyBuffer/100)*100;
		alarmHour.innerHTML = (digit43 < 10)? ("0" + digit43) : digit43;
		alarmMin.innerHTML  = (digit21 < 10)? ("0" + digit21) : digit21;
		timerGraph.setAlarmTime(0);
	}

	// Adjust position of the clock
	if (nowHour < 10)
		clock.style.left = "3px";
	else
		clock.style.left = "7px";

	// Change style of the clock
	if (alarmMode) {
		alarm.style.color = "#ff9900";
		timerGraph.setAlarmMode(true);
	} else {
		alarm.style.color = (keyBuffer == null)? "#999999" : "#667766";
		timerGraph.setAlarmMode(false);
	}
}

// Timer Set
function setTimer(alarmTimeReq) {
	var request;
	var dd = new Date();
	var now = dd.getTime();
	var nowTime = dd.getHours()*60*60*1000 + dd.getMinutes()*60*1000;

	notification.end();

    // Set Alarm Time
    request = parseInt(alarmTimeReq);
	if (request == null)
		request = 0;
	while (request < 0)
		request += 24*60*60*1000;
	while (24*60*60*1000 <= request)
		request -= 24*60*60*1000;
	alarmTime = request;

	// Time to alarm
	zeroHour = Math.floor(now/(60*1000))*60*1000 - nowTime + alarmTime;
	if (alarmTime <= nowTime)
		zeroHour += 24*60*60*1000;

	rewrite();
}

function timerReset() {
	var dd = new Date();
	var nowTime = dd.getHours()*60*60*1000 + dd.getMinutes()*60*1000;

	setTimer(nowTime);
}

// Preset Buttons
function savePreset(key) {
	widgetPref.setValue(alarmTime, key);
	keyBuffer = null;
    rewritePresetBalloon(key, alarmTime);
	rewrite();
	sound.beep();
}

function loadPreset(key) {
	keyBuffer = null;
	setTimer(widgetPref.value(key));
}

function rewritePresetBalloon(key, presetTime) {
	var balloonA = document.getElementById("balloonpreseta");
	var balloonB = document.getElementById("balloonpresetb");
    var balloonText;
    var hour, min;

    hour = Math.floor(presetTime/1000/60/60);
    min = Math.floor((presetTime - hour*60*60*1000)/1000/60);
    balloonText = hour;
    balloonText += ":";
    balloonText += (min < 10)?  ("0" + min) : min;

    switch(key) {
        case "clockA":
            balloonA.innerHTML = balloonText;
            break;
        case "clockB":
            balloonB.innerHTML = balloonText;
            break;
    }
}

// Append a number to keyBuffer
function numImput(num) {
	var keyBufferNow;
	var keyBufferNext;
	var digit43;
	var digit21;

	alarmStop();
	if (keyBuffer == null)
		keyBuffer = 0;

	keyBufferNext = keyBuffer*10 + num;
	keyBuffer = keyBufferNext - (Math.floor(keyBufferNext/10000))*10000;

	digit43 = Math.floor(keyBuffer/100);
	digit21 = keyBuffer - Math.floor(keyBuffer/100)*100;

	while (60 <= digit21) {
		digit21 -= 60;
		digit43++;
	}
	while (24 <= digit43) {
		digit43 -= 24;
	}

    keyBufferNow = keyBuffer;
	setTimer((digit43*60 + digit21)*60*1000);
    keyBuffer = keyBufferNow;
}

// Change jacket
function changeJacket(rev) {
	var newJacket;
	if (!rev) {
		newJacket = widgetPref.value("jacket") + 1;
		if (jacketFiles.length <= newJacket )
			newJacket = 0;
	} else {
		newJacket = widgetPref.value("jacket") - 1;
		if (newJacket < 0)
			newJacket = jacketFiles.length - 1;
	}
	setJacket(newJacket);
}

function setJacket(num) {
    var colorCode;
	var infobutton = document.getElementById("infobutton").object;

	widgetPref.setValue(num, "jacket");
    colorCode = jacketFiles[widgetPref.value("jacket")];

    // Set Images
	document.getElementById("front").style.backgroundImage
		= 'url("Images/jacket' + colorCode + '.png")';
    AlarmWidgetPlugin.setGrowlIcon("Images/Icon" + colorCode + ".png");

    /*
    startButton.setImageDefault("Images/startButton0" + colorCode + ".png");
    startButton.setImageClicked("Images/startButton1" + colorCode + ".png");
    setButtonA.setImageDefault("Images/setButton0" + colorCode + ".png");
    setButtonA.setImageClicked("Images/setButton1" + colorCode + ".png");
    setButtonB.setImageDefault("Images/setButton0" + colorCode + ".png");
    setButtonB.setImageClicked("Images/setButton1" + colorCode + ".png");
    */

	switch (colorCode) {
		case "Gr":
		case "Gl":
		case "Yl":
		case "Wt":
		case "Pk":
			infobutton.setStyle("black", "black");
			break;
		default: 
			infobutton.setStyle("white", "white");
			break;
	}
}

function setAlarmMode(flag) {
	alarmMode = (flag == true);
	widgetPref.setValue(alarmMode, "alarmMode");
	rewrite();
}

// Open website
function openWebsite() {
	if (window.widget)
		widget.openURL(websiteURI);
	else
		window.location.href = websiteURI;
}

// Reverse sides and preferences
function showBack(event) {
	if (window.widget)
		widget.prepareForTransition("ToBack");

	prefPane.setValue(widgetPref.value("jacket"), "jacket");

	document.getElementById("front").style.display = "none";
	document.getElementById("back").style.display = "block";
	display = "back";
	
	if (window.widget)
		setTimeout ("widget.performTransition();", 0);  
}

function showFront(event) {
	if (window.widget)
		widget.prepareForTransition("ToFront");

	document.getElementById("front").style.display = "block";
	document.getElementById("back").style.display = "none";
	display = "front";

	prefPane.apply();
	setTimeout ("rewrite();", 100);

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
}


// ================================================
// EventHandler (keyboard)
// ================================================
function KeyEvents() {
	LOG__("Object: KeyEvents");
	document.addEventListener("keydown",  this.onKeyDown, true);
	document.addEventListener("keypress", this.onKeyPress, true);
	document.addEventListener("keyup",    this.onKeyUp, true);
}

KeyEvents.prototype.onKeyDown = function(theEvent) {
	LOG2__("onKeyDown charCode:" + theEvent.charCode);
	var handled = true;

	if (display == "front") {
		switch(theEvent.charCode) {
			case 63232:  // up
				if (!theEvent.shiftKey)
					inc43Button.keyDown();
				else
					inc43Button.keyDownShift();
				break;
			case 63233:  // down
				if (!theEvent.shiftKey)
					dec43Button.keyDown();
				else
					dec43Button.keyDownShift();
				break;
			case 63235: // right
				if (!theEvent.shiftKey)
					inc21Button.keyDown();
				else
					inc21Button.keyDownShift();
				break;
			case 63234:  // left
				if (!theEvent.shiftKey)
					dec21Button.keyDown();
				else
					dec21Button.keyDownShift();
				break;
			case 32:  // Space bar
				startButton.keyDown();
				break;
			case 8:      // delete
			case 63272:  // del
			case 63289:  // clear
				timerReset();
				break;
			case 116:  // t
			case 84:   // T
				sound.test();
				break;
			case 119:  // w
			case 87:   // W
				openWebsite();
				break;
			case 97:  // a
			case 65:  // A
				setButtonA.keyDown();
				break;
			case 98:  // b
			case 66:  // B
				setButtonB.keyDown();
				break;
			case 97:   // a
			case 65:   // A
			case 98:   // b
			case 66:   // B
			case 115:  // s
			case 83:   // S
			case 9:    // Tab
				break;

			// numbaer input 0 - 9
			case 48:
			case 49:
			case 50:
			case 51:
			case 52:
			case 53:
			case 54:
			case 55:
			case 56:
			case 57:
				numImput(theEvent.charCode - 48);
				break;
			case 13:  // return
			case 3:   // enter
				if (keyBuffer != null) {
					keyBuffer = null;
					rewrite();
				}
				break;
			default:
				handled = false;
				break;
		}
	}

	if (handled) {
		theEvent.stopPropagation();
		theEvent.preventDefault();
	}
}

KeyEvents.prototype.onKeyPress = function(theEvent) {
	LOG2__("onKeyPress charCode:" + theEvent.charCode);
	var handled = true;

	if (display == "front") {
		switch(theEvent.charCode) {
			case 63232:  // up
			case 63233:  // down
			case 63235:  // right
			case 63234:  // left
				break;
			case 63276:  // pageup
				setTimer(alarmTime + 6 * 60 * 60 * 1000);
				break;
			case 63277:  // pagedown
				setTimer(alarmTime - 6 * 60 * 60 * 1000);
				break;
			case 32:  // space
			case 97:  // a
			case 65:  // A
			case 98:  // b
			case 66:  // B
				break;
			default:
				handled = false;
				break;
		}
	}

	if (handled) {
		theEvent.stopPropagation();
		theEvent.preventDefault();
	}
}

KeyEvents.prototype.onKeyUp = function(theEvent) {
	var handled = true;
	LOG2__("onKeyUp charCode:" + theEvent.charCode);

	if (display == "front") {
		switch(theEvent.charCode) {
			case 63232:  // Up
				if (!theEvent.shiftKey)
					inc43Button.keyUp();
				else
					inc43Button.keyUpShift();
				break;
			case 63233:  // Down
				if (!theEvent.shiftKey)
					dec43Button.keyUp();
				else
					dec43Button.keyUpShift();
				break;
			case 63235: // right
				if (!theEvent.shiftKey)
					inc21Button.keyUp();
				else
					inc21Button.keyUpShift();
				break;
			case 63234:  // left
				if (!theEvent.shiftKey)
					dec21Button.keyUp();
				else
					dec21Button.keyUpShift();
				break;
			case 32:  // Space bar
				startButton.keyUp();
				break;
			case 97:  // a
			case 65:  // A
				setButtonA.keyUp();
				break;
			case 98:  // b
			case 66:  // B
				setButtonB.keyUp();
				break;
			case 115:  // s
			case 83:   // S
			case 9:    // Tab
				if (!theEvent.shiftKey)
					changeJacket(false);
				else
					changeJacket(true);
				break;
			default:
				handled = false;
				break;
		}
	}

	if (handled) {
		theEvent.stopPropagation();
		theEvent.preventDefault();
	}
}

// ================================================
// Sound
// ================================================
function Sound() {
	LOG__("Object: Sound");
	if (!(window.widget && AlarmWidgetPlugin))
		LOG__("Cannot load the Widget plugin");
}

// Load the sound file
Sound.prototype.load = function(uri) {
	if (window.widget && AlarmWidgetPlugin)
		AlarmWidgetPlugin.load(uri);
	else
		LOG__("Widget plugin not loaded: loadSound failed.");
}

// Stop playing sound
Sound.prototype.stop = function() {
	if (window.widget && AlarmWidgetPlugin)
		AlarmWidgetPlugin.stop();
}

// Play the sound
Sound.prototype.play = function() {
	if (window.widget && AlarmWidgetPlugin)
		AlarmWidgetPlugin.play(soundRepeat);
	else
		alert("It's Time!");
}

// Test the sound
Sound.prototype.test = function() {
	if (window.widget && AlarmWidgetPlugin)
		AlarmWidgetPlugin.play(1);
}

// Test the beep
Sound.prototype.beep = function() {
	if (window.widget && AlarmWidgetPlugin)
		AlarmWidgetPlugin.beep();
}

// ================================================
// Debugging
// ================================================
function LOG__(arg) {
	return;
	if (window.widget)
		alert(arg);
	else
		if (window.console) window.console.log(arg);
}

function LOG2__(arg) {
	return;
	LOG__(arg);
}

function LOG3__(arg) {
	return;
	LOG__(arg);
}
